@@ -860,6 +860,12 @@ WebAssembly.RuntimeError = function() {};
  * @return {!Promise<{module:WebAssembly.Module, instance:WebAssembly.Instance}>}
  */
 WebAssembly.instantiate = function(moduleObject, importObject) {};
+/**
+ * @param {!Promise<!Response>} source
+ * @param {Object=} importObject
+ * @return {!Promise<{module:WebAssembly.Module, instance:WebAssembly.Instance}>}
+ */
+WebAssembly.instantiateStreaming = function(source, importObject) {};
 /**
  * @param {!BufferSource} bytes
  * @return {!Promise<!WebAssembly.Module>}
