@@ -1,6 +1,7 @@
-from toolchain_profiler import ToolchainProfiler
+from __future__ import print_function
+from .toolchain_profiler import ToolchainProfiler
 import os.path, sys, shutil, time, logging
-import tempfiles, filelock
+from . import tempfiles, filelock
 
 # Permanent cache for dlmalloc and stdlibc++
 class Cache(object):
@@ -28,10 +29,13 @@ class Cache(object):
     self.filelock = filelock.FileLock(self.filelock_name)
 
     if use_subdir:
-      if shared.Settings.WASM_BACKEND:
-        dirname = os.path.join(dirname, 'wasm')
-      else:
+      if not shared.Settings.WASM_BACKEND:
         dirname = os.path.join(dirname, 'asmjs')
+      else:
+        if shared.Settings.EXPERIMENTAL_USE_LLD:
+          dirname = os.path.join(dirname, 'wasm-lld')
+        else:
+          dirname = os.path.join(dirname, 'wasm')
     self.dirname = dirname
     self.debug = debug
     self.acquired_count = 0
@@ -72,8 +76,8 @@ class Cache(object):
     tempfiles.try_delete(self.dirname)
     try:
       open(self.dirname + '__last_clear', 'w').write('last clear: ' + time.asctime() + '\n')
-    except Exception, e:
-      print >> sys.stderr, 'failed to save last clear time: ', e
+    except Exception as e:
+      print('failed to save last clear time: ', e, file=sys.stderr)
     self.filelock = None
     tempfiles.try_delete(self.filelock_name)
     self.filelock = filelock.FileLock(self.filelock_name)
@@ -129,4 +133,8 @@ def chunkify(funcs, chunk_size, DEBUG=False):
       curr = None
     return [''.join([func[1] for func in chunk]) for chunk in chunks] # remove function names
 
-import shared
+try:
+  from . import shared
+except ImportError:
+  # Python 2 circular import compatibility
+  import shared
